"""
Author: Lukas Runt
Email: lrunt@students.zcu.cz
Date: November 30, 2023
Version: 1.0.0
Description: Knowledge base, this module contains all rules that expert system uses
"""


def rule01(nationalities, colors):
    """
    Testing if data meet the rule:
    Angličan žije v červeném domě.
    :param nationalities: nationality permutation
    :param colors: color permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return colors.index('Červená') == nationalities.index('Angličan')


def rule02(nationalities, pets):
    """
    Testing if data meet the rule:
    Švéd chová psy.
    :param nationalities: nationality permutation
    :param pets: pets permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return nationalities.index("Švéd") == pets.index('Pes')


def rule03(nationalities, drinks):
    """
    Testing if data meet the rule:
    Dán pije čaj.
    :param nationalities: nationality permutation
    :param drinks: drink permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return nationalities.index("Dán") == drinks.index("Čaj")


def rule04(colors):
    """
    Testing if data meet the rule:
    Zelený dům je hned vedle bílého domu na levé straně.
    :param colors: color permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return colors.index("Zelená") == (colors.index("Bílá") - 1)


def rule05(colors, drinks):
    """
    Testing if data meet the rule:
    Obyvatel zeleného domu pije kávu.
    :param colors: color permutation
    :param drinks: drinks permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return colors.index("Zelená") == drinks.index("Káva")


def rule06(cigarettes, pets):
    """
    Testing if data meet the rule:
    Ten, co kouří Pall Mall, chová ptáky
    :param cigarettes: cigarettes permutation
    :param pets: pets permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return cigarettes.index("Pall Mall") == pets.index("Pták")


def rule07(colors, cigarettes):
    """
    Testing if data meet the rule:
    Obyvatel žlutého domu kouří Dunhill.
    :param colors: color permutation
    :param cigarettes: cigarettes permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return colors.index("Žlutá") == cigarettes.index("Dunhill")


def rule08(drinks):
    """
    Testing if data meet the rule:
    Ten, co žije ve středním domě, pije mléko.
    :param drinks: drink permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return drinks.index("Mléko") == 2


def rule09(nationalities):
    """
    Testing if data meet the rule:
    Nor žije v prvním domě.
    :param nationalities: nationality permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return nationalities.index("Nor") == 0


def rule10(cigarettes, pets):
    """
    Testing if data meet the rule:
    Ten, co kouří Blend, žije vedle toho, co chová kočky.
    :param cigarettes: cigarettes permutation
    :param pets: color permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return abs(cigarettes.index("Blend") - pets.index("Kočka")) == 1


def rule11(pets, cigarettes):
    """
    Testing if data meet the rule:
    Ten, co chová koně, žije vedle toho, co kouří Dunhill.
    :param cigarettes: cigarettes permutation
    :param pets: color permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return abs(pets.index("Kůň") - cigarettes.index("Dunhill")) == 1


def rule12(cigarettes, drinks):
    """
    Testing if data meet the rule:
    Ten, co kouří Blue Master, pije pivo.
    :param cigarettes: cigarettes permutation
    :param drinks: drink permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return cigarettes.index("Blue Master") == drinks.index("Pivo")


def rule13(nationalities, cigarettes):
    """
    Testing if data meet the rule:
    Němec kouří Prince.
    :param nationalities: nationality permutation
    :param cigarettes: cigarettes permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return nationalities.index("Němec") == cigarettes.index("Prince")


def rule14(nationalities, colors):
    """
    Testing if data meet the rule:
    Nor žije vedle modrého domu.
    :param nationalities: nationality permutation
    :param colors: color permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return abs(nationalities.index("Nor") - colors.index("Modrá")) == 1


def rule15(cigarettes, drinks):
    """
    Testing if data meet the rule:
    Ten, co kouří Blend, má souseda, který pije vodu
    :param cigarettes: cigarettes permutation.
    :param drinks: drink permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return abs(cigarettes.index("Blend") - drinks.index("Voda")) == 1


def check_valid_color(colors):
    """
    Function for checking validity of rules with colors
    :param colors: lists of colors
    :return: if the incoming data meets the rules
    """
    return rule04(colors)


def check_valid_color_nationality(colors, nationalities):
    """
    Function for checking validity of rules with colors and nationalities
    :param colors: lists of colors
    :param nationalities: list of nationalities
    :return: if the incoming data meets the rules
    """
    return rule01(nationalities, colors) and rule04(colors) and rule09(nationalities) and rule14(nationalities, colors)


def check_valid_color_nationality_drink(colors_nationalities, drinks):
    """
    Function for checking validity of rules with colors, nationalities and drinks
    :param colors_nationalities: lists of colors and nationalities
    :param drinks: list of drinks
    :return: if the incoming data meets the rules
    """
    colors = colors_nationalities[0]
    nationalities = colors_nationalities[1]
    return rule01(nationalities, colors) and rule03(nationalities, drinks) and rule04(colors) and \
           rule05(colors, drinks) and rule08(drinks) and rule09(nationalities) and rule14(nationalities, colors)


def check_valid_color_nationality_drink_cigarettes(colors_nationalities_drinks, cigarettes):
    """
    Function for checking validity of rules with colors, nationalities, drinks and cigarettes
    :param colors_nationalities_drinks: lists of colors, nationalities and drinks
    :param cigarettes: list of cigarettes
    :return: if the incoming data meets the rules
    """
    colors = colors_nationalities_drinks[0]
    nationalities = colors_nationalities_drinks[1]
    drinks = colors_nationalities_drinks[2]
    return rule01(nationalities, colors) and rule03(nationalities, drinks) and rule04(colors) and \
           rule05(colors, drinks) and rule07(colors, cigarettes) and rule08(drinks) and rule09(nationalities) and \
           rule12(cigarettes, drinks) and rule13(nationalities, cigarettes) and rule14(nationalities, colors) and \
           rule15(cigarettes, drinks)


def check_validity(colors_nationalities_drinks_cigarettes, pets):
    """
    Function for checking validity of all rules
    :param colors_nationalities_drinks_cigarettes: lists of colors, nationalities, drinks and cigarettes
    :param pets: list of pets
    :return: if the incoming data meets the rules
    """
    colors = colors_nationalities_drinks_cigarettes[0]
    nationalities = colors_nationalities_drinks_cigarettes[1]
    drinks = colors_nationalities_drinks_cigarettes[2]
    cigarettes = colors_nationalities_drinks_cigarettes[3]
    return rule01(nationalities, colors) and rule02(nationalities, pets) and rule03(nationalities, drinks) and \
           rule04(colors) and rule05(colors, drinks) and rule06(cigarettes, pets) and rule07(colors, cigarettes) and \
           rule08(drinks) and rule09(nationalities) and rule10(cigarettes, pets) and rule11(pets, cigarettes) and \
           rule12(cigarettes, drinks) and rule13(nationalities, cigarettes) and rule14(nationalities, colors) \
           and rule15(cigarettes, drinks)
