"""
Author: Lukas Runt
Email: lrunt@students.zcu.cz
Date: November 29, 2023
Version: 1.0.0
Description: Inference module of expert system, infers a solution
"""
import itertools
from BazeZnalosti import *
import BazeDat
from VysvetlovaciModul import print_results
import KomunikacniModul


def app():
    """
    Main functionality of application
    """
    print(KomunikacniModul.welcome_message)
    print(KomunikacniModul.einstein_riddle)
    print(KomunikacniModul.line)
    print(KomunikacniModul.rules)
    print(KomunikacniModul.line)
    print(KomunikacniModul.press_enter, end="")
    input()
    result = solve_einstein()
    if len(result) != 0:
        print_results(result)
    else:
        print(KomunikacniModul.no_solution)
    print(KomunikacniModul.close_program)
    input()


def color_permutations(colors):
    """
    Gets colors permutation that meet the rules
    :param colors: list of colors
    :return: permutations that meet rules
    """
    permutations = list(itertools.permutations(colors, len(colors)))
    useful_permutations = []
    for permutation in permutations:
        if check_valid_color(permutation):
            useful_permutations.append(permutation)
    return useful_permutations


def color_nationality_permutations(color_permutation, nationalities):
    """
    Gets color and nationality permutations that meets rules
    :param color_permutation: list of color permutations that meets rules
    :param nationalities: list of nationalities
    :return: permutations that meet rules
    """
    useful_permutations = []
    for colors in color_permutation:
        nationality_permutations = list(itertools.permutations(nationalities, len(nationalities)))
        for nationalities in nationality_permutations:
            if check_valid_color_nationality(colors, nationalities):
                useful_permutations.append([colors, nationalities])
    return useful_permutations


def color_nationality_drink_permutations(color_nationality_permutation, drinks):
    """
    Gets color, nationality and drink permutations that meets rules
    :param color_nationality_permutation: list of color and nationality permutations that meets rules
    :param drinks: list of drinks
    :return: permutations that meet rules
    """
    useful_permutation = []
    for color_nationality in color_nationality_permutation:
        drinks_permutation = list(itertools.permutations(drinks, len(drinks)))
        for drinks in drinks_permutation:
            if check_valid_color_nationality_drink(color_nationality, drinks):
                useful_permutation.append([color_nationality[0], color_nationality[1], drinks])
    return useful_permutation


def color_nationality_drink_cigarettes_permutations(color_nationality_drink_permutation, cigarettes):
    """
    Gets color, nationality, drink and cigarettes permutations that meets rules
    :param color_nationality_drink_permutation: list of color, nationality and drink permutations that meets rules
    :param cigarettes: list of cigarettes
    :return: permutations that meet rules
    """
    useful_permutation = []
    for color_nationality_drink in color_nationality_drink_permutation:
        cigarettes_permutation = list(itertools.permutations(cigarettes, len(cigarettes)))
        for cigarettes in cigarettes_permutation:
            if check_valid_color_nationality_drink_cigarettes(color_nationality_drink, cigarettes):
                useful_permutation.append \
                    ([color_nationality_drink[0], color_nationality_drink[1], color_nationality_drink[2], cigarettes])
    return useful_permutation


def get_results(color_nationality_drink_cigarettes_permutation, pets):
    """
    Gets the results that meet rules
    :param color_nationality_drink_cigarettes_permutation: list of color, nationality, drink and cigarettes permutations that meets rules
    :param pets: list of pets
    :return: get solutions of einstein riddle
    """
    results = []
    for color_nationality_drink_cigarettes in color_nationality_drink_cigarettes_permutation:
        pets_permutation = list(itertools.permutations(pets, len(pets)))
        for pets in pets_permutation:
            if check_validity(color_nationality_drink_cigarettes, pets):
                results.append \
                    ([BazeDat.houses, color_nationality_drink_cigarettes[0], color_nationality_drink_cigarettes[1],
                      color_nationality_drink_cigarettes[2], color_nationality_drink_cigarettes[3], pets])
    return results


def solve_einstein():
    """
    Function for solving the einstein problem
    :return: results
    """
    possible_color_permutations = color_permutations(BazeDat.colors)
    possible_color_nationality_permutations = color_nationality_permutations(possible_color_permutations,
                                                                             BazeDat.nations)
    possible_color_nationality_drink_permutations = color_nationality_drink_permutations \
        (possible_color_nationality_permutations, BazeDat.drinks)
    possible_color_nationality_drink_cigarettes_permutations = color_nationality_drink_cigarettes_permutations \
        (possible_color_nationality_drink_permutations, BazeDat.cigarettes)
    results = get_results(possible_color_nationality_drink_cigarettes_permutations, BazeDat.pets)
    return results
