"""
Author: Lukas Runt
Email: lrunt@students.zcu.cz
Date: November 30, 2023
Version: 1.0.0
Description: Knowledge base, this module contains all rules that expert system uses
"""


def rule01(nationalities, colors):
    """
    Testing if data meet the rule:
    Angličané žijí v červeném domě.
    :param nationalities: nationality permutation
    :param colors: color permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return colors.index('Červená') == nationalities.index('Angličané')


def rule02(nationalities, pets):
    """
    Testing if data meet the rule:
    Španělé mají psa.
    :param nationalities: nationality permutation
    :param pets: pets permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return nationalities.index("Španělé") == pets.index('Pes')


def rule03(nationalities):
    """
    Testing if data meet the rule:
    Norové žijí v prvém domě zleva.
    :param nationalities: nationality permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return nationalities.index("Norové") == 0


def rule04(colors, cigarettes):
    """
    Testing if data meet the rule:
    Obyvatelé žlutého domu kouří sparty.
    :param colors: color permutation
    :param cigarettes: cigarettes permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return colors.index("Žlutá") == cigarettes.index("Sparta")


def rule05(cigarettes, pets):
    """
    Testing if data meet the rule:
    Muž, který kouří Chesterfieldky, žije v domě vedle domu rodiny, která chová lišku.
    :param cigarettes: cigarettes permutation
    :param pets: pets permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return abs(cigarettes.index("Chesterfield") - pets.index("Liška")) == 1


def rule06(nationalities, colors):
    """
    Testing if data meet the rule:
    Norové žijí v modrém domě.
    :param nationalities: nationalities permutation
    :param colors: colors permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return nationalities.index("Norové") == colors.index("Modrá")


def rule07(cigarettes, pets):
    """
    Testing if data meet the rule:
    Kuřák Winstonek chová šneky.
    :param pets: pets permutation
    :param cigarettes: cigarettes permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return cigarettes.index("Winston") == pets.index("Šnek")


def rule08(cigarettes, drinks):
    """
    Testing if data meet the rule:
    Kuřák Lucky-Strike pije pomerančový džus.
    :param cigarettes: cigarettes permutation
    :param drinks: drink permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return cigarettes.index("Lucky-Strike") == drinks.index("Pomerančový džus")


def rule09(nationalities, drinks):
    """
    Testing if data meet the rule:
    Ukrajinci pijí čaj.
    :param nationalities: nationality permutation
    :param drinks: drinks permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return nationalities.index("Ukrajinci") == drinks.index("Čaj")


def rule10(nationalities, cigarettes):
    """
    Testing if data meet the rule:
    Japonci kouří cigarety značky Parliaments.
    :param cigarettes: cigarettes permutation
    :param nationalities: color permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return cigarettes.index("Parliaments") == nationalities.index("Japonci")


def rule11(pets, cigarettes):
    """
    Testing if data meet the rule:
    V domě vedle domu s koněm se kouří sparty.
    :param cigarettes: cigarettes permutation
    :param pets: color permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return abs(pets.index("Kůň") - cigarettes.index("Sparta")) == 1


def rule12(colors, drinks):
    """
    Testing if data meet the rule:
    V zeleném domě pijí kávu.
    :param colors: colors permutation
    :param drinks: drink permutation
    :return: True - meet the rule, False - do not meet the rule
    """
    return colors.index("Zelená") == drinks.index("Káva")


def rule13(colors):
    """
    Testing if data meet the rule:
    Zelený dům je bezprostředně napravo od bílého domu.
    :param colors: colors permutations
    :return: True - meet the rule, False - do not meet the rule
    """
    return colors.index("Bílá") == (colors.index("Zelená") - 1)


def rule14(drinks):
    """
    Testing if data meet the rule:
    V prostředním domu pijí mléko.
    :param drinks: drinks permutations
    :return: True - meet the rule, False - do not meet the rule
    """
    return drinks.index("Mléko") == 2

def check_valid_color(colors):
    """
    Function for checking validity of rules with colors
    :param colors: lists of colors
    :return: if the incoming data meets the rules
    """
    return rule13(colors)


def check_valid_color_nationality(colors, nationalities):
    """
    Function for checking validity of rules with colors and nationalities
    :param colors: lists of colors
    :param nationalities: list of nationalities
    :return: if the incoming data meets the rules
    """
    return rule01(nationalities, colors) and rule03(nationalities) and rule06(nationalities, colors) and rule13(colors)


def check_valid_color_nationality_drink(colors_nationalities, drinks):
    """
    Function for checking validity of rules with colors, nationalities and drinks
    :param colors_nationalities: lists of colors and nationalities
    :param drinks: list of drinks
    :return: if the incoming data meets the rules
    """
    colors = colors_nationalities[0]
    nationalities = colors_nationalities[1]
    return rule01(nationalities, colors) and rule03(nationalities) and rule06(nationalities, colors) and \
           rule09(nationalities, drinks) and rule12(colors, drinks) and rule13(colors) and rule14(drinks)


def check_valid_color_nationality_drink_cigarettes(colors_nationalities_drinks, cigarettes):
    """
    Function for checking validity of rules with colors, nationalities, drinks and cigarettes
    :param colors_nationalities_drinks: lists of colors, nationalities and drinks
    :param cigarettes: list of cigarettes
    :return: if the incoming data meets the rules
    """
    colors = colors_nationalities_drinks[0]
    nationalities = colors_nationalities_drinks[1]
    drinks = colors_nationalities_drinks[2]
    return rule01(nationalities, colors) and rule03(nationalities) and rule04(colors, cigarettes) and \
           rule06(nationalities, colors) and rule08(cigarettes, drinks) and rule09(nationalities, drinks) and\
           rule10(nationalities, cigarettes) and rule12(colors, drinks) and rule13(colors)\
           and rule14(drinks)


def check_validity(colors_nationalities_drinks_cigarettes, pets):
    """
    Function for checking validity of all rules
    :param colors_nationalities_drinks_cigarettes: lists of colors, nationalities, drinks and cigarettes
    :param pets: list of pets
    :return: if the incoming data meets the rules
    """
    colors = colors_nationalities_drinks_cigarettes[0]
    nationalities = colors_nationalities_drinks_cigarettes[1]
    drinks = colors_nationalities_drinks_cigarettes[2]
    cigarettes = colors_nationalities_drinks_cigarettes[3]
    return rule01(nationalities, colors) and rule02(nationalities, pets) and rule03(nationalities) and \
           rule04(colors, cigarettes) and rule05(cigarettes, pets) and rule06(nationalities, colors) and\
           rule07(cigarettes, pets) and rule08(cigarettes, drinks) and rule09(nationalities, drinks) and\
           rule10(nationalities, cigarettes) and rule11(pets, cigarettes) and rule12(colors, drinks) and\
           rule13(colors) and rule14(drinks)
