"""
Author: Lukas Runt
Email: lrunt@students.zcu.cz
Date: November 29, 2023,
Version: 1.0.0
Description: Knowledge base, module contains all definitions of node and AND/OR graph
"""

import BazeDat


class TreeNode:
    """
    Represents node of AND/OR graph

    Attributes
    ----------
    name: str
        name - value of the node
    type_of_node: str
        type of the node AND, OR or list
    children: List
        children of the node
    """
    def __init__(self, name: str, type_of_node: str):
        self.type_of_node = None
        self.set_type(type_of_node)
        self.children = []
        self.name = name

    def set_type(self, type_of_node):
        """
        Setting type of the node and control if the type is valid
        :param type_of_node: type of the node
        """
        if type_of_node in BazeDat.typeOfNode:
            self.type_of_node = type_of_node
        else:
            raise ValueError(f"Type {type_of_node} is not a know type.")

    def add_children(self, children):
        """
        Adding children dependency
        :param children: children of the node
        """
        self.children.append(children)


class AndORGraph:
    """
    Represents AND/OR graph

    Attributes
    ----------
    root: node
        root node of the graph
    nodes: List
        all nodes of the graph
    """
    def __init__(self, root):
        self.root = root
        self.nodes = []

    def create_tree(self, data):
        """
        Method creates AND/OR graph from the data
        :param data: data to creating an AND/OR graph
        """
        for i in range(len(data)):
            self.add_node(data[i])
        self.root = self.get_node_from_array(self.root)

    def node_exists_in_array(self, name: str) -> bool:
        """
        Returns if the node exists in the graph
        :param name: name of the node
        :return: True - node exists, False - node do not exist
        """
        for node in self.nodes:
            if node.name == name:
                return True
        return False

    def get_node_from_array(self, name: str) -> TreeNode:
        """
        Returns a node from the graph
        :param name: name of the node
        :return: node of the AND/OR graph or None
        """
        for node in self.nodes:
            if node.name == name:
                return node
        return None

    def get_array_by_name(self, name: str):
        """
        Getting data from the array
        :param name: name of the data
        :return: data or None
        """
        for item in BazeDat.data:
            if item[0] == name:
                return item
        return None

    def add_node(self, node_data: []):
        """
        Adding node to the AND/OR graph
        :param node_data: data of the node
        """
        # If node do not exist
        if not self.node_exists_in_array(node_data[0]):
            node = TreeNode(node_data[0], node_data[1])
            # Children nodes
            for children in node_data[2]:
                if not self.node_exists_in_array(children):
                    self.add_node(self.get_array_by_name(children))
                node.add_children(self.get_node_from_array(children))
            self.nodes.append(node)
