"""
Author: Lukas Runt
Email: lrunt@students.zcu.cz
Date: November 29, 2023
Version: 1.0.0
Description: Explanation module of expert system, getting and printing result to the user
"""

import KomunikacniModul


def draw_tree(node, level=0, prefix="", last_sibling=True):
    """
    Method prints a tree to the console
    :param node: node of the AND/OR graph
    :param level: level of the node
    :param prefix: the size of indentation
    :param last_sibling: if is the node last in the graph
    """
    if node:
        if node.type_of_node == "":
            print(" " * (level * 4) + prefix + ("└── " if last_sibling else "├── ") + f"{node.name}")
        else:
            print(" " * (level * 4) + prefix + (
                "└── " if last_sibling else "├── ") + f"{node.name} - {node.type_of_node}")
        for child in node.children:
            draw_tree(child, level + 1, "    ", child == node.children[-1])


def print_children(graph, node: str):
    """
    Function prints children of the node and print that in nature language
    :param graph: AND/OR graph
    :param node: node what children will be printed
    """
    graph_node = graph.get_node_from_array(node)
    num_of_children = len(graph_node.children)
    if node != "Boloňská omáčka":
        text = f"{node} {KomunikacniModul.contains_plural} "
    else:
        text = f"{node} {KomunikacniModul.contains_singular} "
    for i in range(num_of_children):
        if i == num_of_children - 1:
            text += f"{KomunikacniModul.inflection_of_words[graph_node.children[i].name]}."
        elif i == num_of_children - 2:
            text += f"{KomunikacniModul.inflection_of_words[graph_node.children[i].name]} a "
        else:
            text += f"{KomunikacniModul.inflection_of_words[graph_node.children[i].name]}, "
    print(text)
    print(KomunikacniModul.enter_to_continue)
    input()
