"""
Description: Tento modul řídí běh znalostního systému.
Author: Václav Ovsík, ovsikv@students.zcu.cz
Version: 1.0
Date: 12.12.2023
"""
import copy

import baze_dat
import baze_znalosti
import komunikacni_modul
import vysvetlovaci_modul


def start():
    # tato metoda spousti znalostni system
    komunikacni_modul.start_info()
    solve(baze_dat.houses, baze_dat.remaining)
    komunikacni_modul.end_info()


def solve(houses, remaining):
    """
    Hleda reseni problemu pomoci rekurze. Postupne zkousi pridavat moznosti do vysledneho seznamu a testuje
    zda stale plati zadana pravidla.
    :param houses: vysledny seznam
    :param remaining: zbyvajici moznosti
    :return: True pokud jsou s aktualnim vyslednym seznamem zadana pravidla splnena
             False v opacnem pripade
    """
    new_remaining = copy.deepcopy(remaining)
    # pokud zbyvaji moznosti barev, zkusime jednu pridat do vysledku a otestujeme zda pravidla stale plati
    if len(remaining.colors) != 0:
        c_to_add = new_remaining.colors.pop(0)
        for i in range(len(houses)):
            if houses[i].color is not None:
                continue
            houses_c = copy.deepcopy(houses)
            houses_c[i].color = c_to_add
            if baze_znalosti.verify_rules(houses_c):
                # test pravidel, pokud projde, ukoncime rekurzi
                if solve(houses_c, new_remaining):
                    return True
    # pokud zbyvaji moznosti narodnosti, zkusime jednu pridat do vysledku a otestujeme zda pravidla stale plati
    elif len(remaining.nationalities) != 0:
        n_to_add = new_remaining.nationalities.pop(0)
        for i in range(len(houses)):
            if houses[i].nationality is None:
                houses_c = copy.deepcopy(houses)
                houses_c[i].nationality = n_to_add
                if baze_znalosti.verify_rules(houses_c):
                    # test pravidel, pokud projde, ukoncime rekurzi
                    if solve(houses_c, new_remaining):
                        return True
    # pokud zbyvaji moznosti napoju, zkusime jednu pridat do vysledku a otestujeme zda pravidla stale plati
    elif len(remaining.drinks) != 0:
        d_to_add = new_remaining.drinks.pop(0)
        for i in range(len(houses)):
            if houses[i].drink is not None:
                continue
            houses_c = copy.deepcopy(houses)
            houses_c[i].drink = d_to_add
            if baze_znalosti.verify_rules(houses_c):
                # test pravidel, pokud projde, ukoncime rekurzi
                if solve(houses_c, new_remaining):
                    return True
    # pokud zbyvaji moznosti cigaret, zkusime jednu pridat do vysledku a otestujeme zda pravidla stale plati
    elif len(remaining.cigarettes) != 0:
        c_to_add = new_remaining.cigarettes.pop(0)
        for i in range(len(houses)):
            if houses[i].cigarette is not None:
                continue
            houses_c = copy.deepcopy(houses)
            houses_c[i].cigarette = c_to_add
            if baze_znalosti.verify_rules(houses_c):
                # test pravidel, pokud projde, ukoncime rekurzi
                if solve(houses_c, new_remaining):
                    return True
    # pokud zbyvaji moznosti zvirat, zkusime jednu pridat do vysledku a otestujeme zda pravidla stale plati
    elif len(remaining.pets) != 0:
        p_to_add = new_remaining.pets.pop(0)
        # del new_remaining.pets[0]
        for i in range(len(houses)):
            if houses[i].pet is not None:
                continue
            houses_c = copy.deepcopy(houses)
            houses_c[i].pet = p_to_add
            if baze_znalosti.verify_rules(houses_c):
                # test pravidel, pokud projde, ukoncime rekurzi
                if solve(houses_c, new_remaining):
                    return True

    # nezbyvaji zadne moznosti, coz znamena ze jsme nasli vysledek
    else:
        vysvetlovaci_modul.print_result(houses)
        return True
